import pygame
from random import randint

# global game jam 2020 entry: random broken devices
# v1 by y-1

# classes
PORTRADIUS=10

class Port:

  def __init__(self,x,y): # auto links to part below
    self.parentPart=findPartAt(x,y) # Part
    self.childPart=None # Part or None
    self.x=x # local
    self.y=y # local
    if self.parentPart!=None:
      self.parentPart.ports+=[self]
      self.x-=self.parentPart.x # local
      self.y-=self.parentPart.y # local

  def isBelow(self,x,y):
    if self.parentPart!=None:
      print "error: parentless port"
      return False
    if x>self.parentPart.x+self.x-PORTRADIUS and x<self.parentPart.x+self.x+PORTRADIUS and y>self.parentPart.y+self.y-PORTRADIUS and y<self.parentPart.y+self.y+PORTRADIUS:
      return True
    return False

  def draw(self):
    pygame.draw.rect(s,(120,120,120),(self.parentPart.x+self.x-PORTRADIUS  ,self.parentPart.y+self.y-PORTRADIUS  ,PORTRADIUS*2,PORTRADIUS*2))
    pygame.draw.rect(s,( 20, 20, 20),(self.parentPart.x+self.x-PORTRADIUS/2,self.parentPart.y+self.y-PORTRADIUS  ,PORTRADIUS  ,PORTRADIUS*2))
    pygame.draw.rect(s,( 20, 20, 20),(self.parentPart.x+self.x-PORTRADIUS  ,self.parentPart.y+self.y-PORTRADIUS/2,PORTRADIUS*2,PORTRADIUS  ))


class Part:

  def __init__(self,x,y,noPcb,isBroken): # auto links to part below by creating a new port on the part below
    self.parentPort=None
    self.ports=[]
    self.x=x # global plug position
    self.y=y # global plug position
    self.x0=0 # local pcb start
    self.x1=0 # local pcb end
    self.y0=0 # local pcb start
    self.y1=0 # local pcb end
    self.rgb=(randint(20,60),randint(80,220),randint(20,60))
    self.isBroken=isBroken

    if noPcb==False:
      PCB_MIN=50
      PCB_MAX=250
      deviceCenterX=deviceX+deviceW/2
      deviceCenterY=deviceY+deviceH/2
      if y<deviceCenterY:
        self.y0=PORTRADIUS # local pcb start
        self.y1=randint(PORTRADIUS+PCB_MIN,PORTRADIUS+PCB_MAX) # local pcb end
      else:
        self.y0=-randint(PORTRADIUS+PCB_MIN,PORTRADIUS+PCB_MAX) # local pcb end
        self.y1=-PORTRADIUS # local pcb start
      if y<deviceCenterX:
        self.x0=-PORTRADIUS # local pcb start
        self.x1=randint(PORTRADIUS+PCB_MIN,PORTRADIUS+PCB_MAX) # local pcb end
      else:
        self.x0=-randint(PORTRADIUS+PCB_MIN,PORTRADIUS+PCB_MAX) # local pcb end
        self.x1=PORTRADIUS # local pcb start

    port=findEmptyPortAt(x,y)
    if port!=None:
      self.link(port)
    else:
      part=findPartAt(x,y)
      if part!=None:
        port=Port(x,y)
        self.link(port)

  def link(self,port):
    if port!=None:
      self.parentPort=port
      port.childPart=self
    else:
      if self.parentPort!=None:
        self.parentPort.childPart=None
      self.parentPort=None

  def unlink(self):
    if self.parentPort!=None:
      self.parentPort.childPart=None
      self.parentPort=None

  def isBelow(self,x,y):
    if x>self.x+self.x0 and x<self.x+self.x1 and y>self.y+self.y0 and y<self.y+self.y1:
      return True
    return False

  def moveTo(self,x,y):
    dx=x-self.x
    dy=y-self.y
    self.x=x
    self.y=y
    for po in self.ports:
      pa=po.childPart
      if pa!=None:
        px=pa.x+dx
        py=pa.y+dy
        pa.moveTo(px,py)

  def draw(self):
    pygame.draw.rect(s,self.rgb,(self.x+self.x0,self.y+self.y0,self.x1-self.x0,self.y1-self.y0))
    pygame.draw.rect(s,(20,20,20),(self.x+self.x0,self.y+self.y0,self.x1-self.x0,self.y1-self.y0),2)
    pygame.draw.rect(s,(20,20,20),(self.x-PORTRADIUS,self.y-PORTRADIUS,PORTRADIUS*2,PORTRADIUS*2))
    if self.isBroken:
      pygame.draw.line(s,(220,220,220),(self.x-PORTRADIUS,self.y+PORTRADIUS),(self.x+PORTRADIUS,self.y-PORTRADIUS))
      pygame.draw.line(s,(220,220,220),(self.x,self.y),(self.x+PORTRADIUS,self.y+PORTRADIUS))
    if self.parentPort!=None:
      pygame.draw.rect(s,(80,80,80),(self.x-PORTRADIUS/2,self.y-PORTRADIUS/2,PORTRADIUS,PORTRADIUS))
    for po in self.ports:
      po.draw()

def findEmptyPortAt(x,y):
  result=None
  for pa in allParts:

    for po in pa.ports:
      if po.isBelow(x,y):
        result=p
  return result

def findPartAt(x,y):
  result=None
  for pa in allParts:
    if pa.isBelow(x,y):
      result=pa
  return result

# init application
pygame.init()
w=800
h=600
s=pygame.display.set_mode((w,h))
pygame.display.set_caption('random broken devices')

# init level
deviceX=50
deviceY=50
deviceW=400
deviceH=300

allParts=[]
allPartsN=10
for i in range(allPartsN):
  pa=Part(randint(deviceX,deviceX+deviceW),randint(deviceY,deviceY+deviceH),False,False) # ,noPcb,isBroken)
  allParts+=[pa]

# init mouse
grabFirstFrame=1
grabPart=None
grabPartX0=0
grabPartY0=0
grabX0=0
grabY0=0

# main loop
exit=False
while not exit:

  # read input
  mx,my=pygame.mouse.get_pos()
  mc=pygame.mouse.get_pressed()

  # input: drop part
  if mc==(0,0,0):
    po=findEmptyPortAt(mx,my)
    if po!=None:
      grabPart.link(po)
    # ...
    grabFirstFrame=1
    grabPart=None

  # input: move part
  if mc!=(0,0,0) and grabPart!=None:
    grabPart.moveTo(grabPartX0-grabX0+mx,grabPartY0-grabY0+my)

  # input: grab new part
  if mc!=(0,0,0) and grabPart==None and grabFirstFrame==1:
    grabFirstFrame=0
    grabPart=findPartAt(mx,my)
    if grabPart!=None:
      grabPartX0=grabPart.x
      grabPartY0=grabPart.y
      grabX0=mx
      grabY0=my
      grabPart.unlink()

  # background
  s.fill((40,40,40))
  pygame.draw.rect(s,(200,200,200),(deviceX,deviceY,deviceW,deviceH))

  # draw parts in z order
  for pa in allParts:
    pa.draw()

  # draw cursor
  c=(0,128,0)
  if mc!=(0,0,0):
    c=(255,0,0)
  pygame.draw.line(s,c,(mx, 0),(mx, h))
  pygame.draw.line(s,c,( 0,my),( w,my))

  # ...
  pygame.display.flip()
  for event in pygame.event.get():
    if event.type == pygame.QUIT:
      exit=True

# exit
pygame.quit()

